#!/bin/bash

#query mage database for installed packages
# $Header: /home/cvsd/magellan-cvs/magellan-src/mage/usr/lib/mage/magequery.sh,v 1.13 2008-10-05 10:33:04 niro Exp $

print_usage()
{
	echo "$(basename $0 .sh) querys the mage database for installed packages."
	echo
	echo "    -n NAME   searches for name NAME"
	echo "    -v VER    searches for version VER, needs -n"
	echo "    -b BUILD  searches for build number BUILD, needs -n -v"
	echo "    -c CAT    shows all packages of given categorie"
	echo "    -i        shows an inventory of all installed packages"
	echo "    -f PATH   searches package names in given path"
	echo
	echo " Examples:"
	echo
	echo "    $(basename $0 .sh) -n xorg -v 6.8.0 -b r1"
	echo "          searches for package xorg-6.8.0-r1"
	echo
}

# default args:
GET_INVENTORY=false
SEARCH_ONLY_PATH=false
SEARCH_ONLY_CAT=false

while getopts "n:v:b:f:c:hi-" opt
do
	case "${opt}" in
		n)
			S_PNAME="${OPTARG}"
			;;

		v)
			S_PVER="${OPTARG}"
			;;

		b)
			S_PBUILD="${OPTARG}"
			;;
		i)
			GET_INVENTORY="true"
			;;
		f)
			SEARCH_ONLY_PATH="true"
			SEARCH_PATH="${OPTARG}"
			;;
		c)
			SEARCH_ONLY_CAT="true"
			SEARCH_CAT="${OPTARG}"
			;;
		h)
			print_usage
			exit 0
			;;

		-) break
			;;

		*)
			print_usage
			exit 1
			;;
	esac
done
shift $(($OPTIND - 1))

source /etc/mage.rc.global
source /etc/mage.rc
RETVAL=1

if [[ ${SEARCH_ONLY_PATH} = true ]]
then
	unset S_PACKAGES
	unset i pkg
	for i in $(fgrep -rl "${SEARCH_PATH}" ${INSTALLDB})
	do
		# print categorie and pkgname
		pkg="$(basename ${i%/*/*})/$(basename ${i%/*})"
		if [[ -z $(echo ${S_PACKAGES} | grep ${pkg}) ]]
		then
			S_PACKAGES="${S_PACKAGES} ${pkg}"
		fi
	done

	# show packages
	for pkg in ${S_PACKAGES}
	do
		echo "${pkg}"
	done

	exit 0
fi

if [[ ${SEARCH_ONLY_CAT} = true ]]
then
	# no packages of SEARCH_CAT are installed
	[[ ! -d ${MROOT}${INSTALLDB}/${SEARCH_CAT} ]] && exit 1

	for i in ${INSTALLDB}/${SEARCH_CAT}/*
	do
		# print categorie and pkgname
		echo "$(basename ${i%/*})/$(basename ${i})"
	done

	exit 0
fi

if [[ ${GET_INVENTORY} = true ]]
then
	for package in $(find ${MROOT}${INSTALLDB} -mindepth 2 -maxdepth 2 -type d -printf "%h,%f\n" | sort)
	do
		pcat="$(basename $(echo ${package} | cut -d, -f1))"
		pname="$(echo ${package} | cut -d, -f2)"
		if [ -z "${invlist}" ]
		then
			invlist="${pcat}/${pname}"
		else
			invlist="${invlist}
${pcat}/${pname}"
		fi
	done

	# now show the list
	echo "${invlist}"
	exit 0
fi

for i in ${MROOT}${INSTALLDB}/*/*
do
	INST_PNAME=no
	INST_PVER=no
	INST_PBUILD=no

	x=$(basename ${i})
	PNAME=${x%-*-*}
	PVER=$(echo ${x#${PNAME}-}| cut -d- -f1)
	PBUILD=$(echo ${x#${PNAME}-}| cut -d- -f2)

	if [[ ${PNAME} == ${S_PNAME} ]]
	then
		INST_PNAME=yes

		if [ -n "${S_PVER}" ]
		then
			if [[ ${PVER} == ${S_PVER} ]]
			then
				INST_PVER=yes
			fi
			
			if [ -n "${S_PBUILD}" ]
			then
				if [[ ${PBUILD} == ${S_PBUILD} ]]
				then
					INST_PBUILD=yes
				fi
			fi
		fi

		# search for pname only
		if [ -n "${S_PNAME}" -a -z "${S_PVER}" -a -z "${S_PBUILD}" ]
		then
			if [ "${INST_PNAME}" = yes ]
			then
				echo "${S_PNAME} is installed [ ${PNAME}-${PVER}-${PBUILD} ]"
				RETVAL=0
				break
			fi
		fi

		# search for pname and pver
		if [ -n "${S_PNAME}" -a -n "${S_PVER}" -a -z "${S_PBUILD}" ]
		then
			if [ "${INST_PNAME}" = "yes" -a "${INST_PVER}" = "yes" ]
			then
				echo "${S_PNAME}-${S_PVER} is installed [ ${PNAME}-${PVER}-${PBUILD} ]"
				RETVAL=0
				break
			fi
		fi

		# search for pname, pver and pbuild
		if [ -n "${S_PNAME}" -a -n "${S_PVER}" -a -n "${S_PBUILD}" ]
		then
			if [ "${INST_PNAME}" = "yes" -a "${INST_PVER}" = "yes"  -a "${INST_PBUILD}" = "yes" ]
			then
				echo "${S_PNAME}-${S_PVER}-${S_PBUILD} is installed [ ${PNAME}-${PVER}-${PBUILD} ]"
				RETVAL=0
				break
			fi
		fi
	fi
done

exit ${RETVAL}
